Program Klimakammer;

{$NOSHADOW}
{ $WG}                     {global Warnings off}

Device = mega16, VCC=5;
{ $BOOTRST $01C00}         {Reset Jump to $01C00}

Import SysTick, ADCPort, LCDPort, SoftPWM, SwitchPort_G;

From System Import Processes;


Define
  ProcClock      = 16000000;       {Hertz}
  SysTick        = 10;             {msec}
  StackSize      = $0100, iData;
  FrameSize      = $0100, iData;

  ADCChans       = [1..2], iData;
  ADCPresc       = 64;

  LCDport        = PortC;
  LCDtype        = 44780;                   { DIP-LCD = 44780                  }
  LCDrows        = 4;                       { rows                             }
  LCDcolumns     = 20;                      { columns per line                 }
  
  SoftPWMport    = PortD;                   { use PortD for PWM output         }
  SoftPWMtimer   = Timer2,10;               { use timer2, PWM cycle time msec  }
  SoftPWMres     = 100;                     { PWM resolution is 100 points     }
  SoftPWMchans   = 3, 5;                    { 3 channels, bit5 is the first bit}
  
  SwitchPort_G   = [TempMinus,PinB,0]
                   [TempPlus,PinB,1]
                   [TimeMinus,PinB,2]
                   [TimePlus,PinB,3]
                   [OnOff,PinB,4], %00000000;
  PolarityP_G    = %00000000;
  SwitchPMask_G  = %00001111;
  
  Scheduler      = iData;
Implementation

{$IDATA}

{--------------------------------------------------------------}
{ Type Declarations }

Type


{--------------------------------------------------------------}
{ Const Declarations }
Const
  KTYLookUp   : Array[1..(20*4)+3] of Integer = 'Klimakammer_KTY.crvg';
{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}
Var
  RTemp       : Integer;
  ITemp       : Integer;
  Speed       : Byte;
  H1          : Byte;
  H2          : Byte;
  ADCAct      : Boolean;
  TSoll       : Integer;
  HeatTime    : Byte;
  HeatSec     : Word;
  NewTime     : Boolean;
  TimeStart   : Boolean;
  StopAuto    : Boolean;
  ProgRun     : Boolean;
  RunStat     : Byte;
  DiffTemp    : Integer;
  DiffIR      : Integer;

{--------------------------------------------------------------}
{ functions }
Procedure HeatControl1(Heat1:Byte);
  Begin
    If ((Heat1 in [0..100]) and (RTemp<110))
      then
        SoftPWMArr[1]:=Heat1;
        H1:=Heat1;
      else
        SoftPWMArr[1]:=0;
        H1:=0;
      EndIf;
  End HeatControl1;


Procedure HeatControl2(Heat2:Byte);
  Begin
    If Heat2 in [0..100]
      then
        SoftPWMArr[2]:=Heat2;
        H2:=Heat2;
      else
        SoftPWMArr[2]:=0;
        H2:=0;
      EndIf;
  End HeatControl2;
  
  
Procedure FanControl(FSpeed:Byte);
  Begin
    If FSpeed in [0..100]
      then
        SoftPWMArr[2]:=FSpeed;
        Speed:=FSpeed;
      else
        SoftPWMArr[2]:=0;
        Speed:=0;
      EndIf;
  End FanControl;

  
Process ADCActiv(30,30:iData);
  Begin
    ADCAct:=true;
    Sleep(ADCActiv,50);
  End ADCActiv;
  
  
Process Reg_Temp(30,30:iData);
  Begin
    Case DiffTemp of
           0:HeatControl1(0);|
           1:HeatControl1(60);|
           2:HeatControl1(80);|
       3..99:HeatControl1(100);|
      Else HeatControl1(0);
      EndCase;
    Sleep(Reg_Temp,50);
  End Reg_Temp;
  
  
Process Fan(30,30:iData);
  Begin
    If ITemp>30
      then
        Case DiffIR of
          -5.. 4:FanControl(40);|
           5..20:FanControl(20+(Lo(DiffIR)*4));|
          21..99:FanControl(100);|
          Else FanControl(100);
          EndCase;
      else
        Case DiffIR of
          -5.. 4:FanControl(0);|
           5..20:FanControl(20+(Lo(DiffIR)*4));|
          21..99:FanControl(100);|
          Else FanControl(0);
          EndCase;
      EndIf;
  End Fan;
  
  
Process Timer(30,30:iData);
  Begin
    If HeatSec>0
      then
        Dec(HeatSec);
      else
        StopAuto:=true;
        Suspend(Timer);
      EndIf;
    NewTime:=true;
    Sleep(Timer,96);
  End Timer;


Procedure ShowRun;
  Begin
    LCDXY(1,3);
    Case RunStat of
      0:Write(LCDOut,'-');|
      1:Write(LCDOut,#1);|
      2:Write(LCDOut,'|');|
      3:Write(LCDOut,'/');|
      EndCase;
    If RunStat<3
      then
        Inc(RunStat);
      else
        RunStat:=0;
      EndIf;
  End ShowRun;
  
  
Procedure ShowStatus;
  Var T1, T2 : Integer;
  Begin
    T1:=Integer(GetADC(1));                  { Innenraum                       }
    T2:=Integer(GetADC(2));                  { Heizung                         }
    If T1 in [52..963]
      then
        InterpolY(@KTYLookUp,T1,ITemp);
      else
        RTemp:=300;
      EndIf;
    If T2 in [52..963]
      then
        InterpolY(@KTYLookUp,T2,RTemp);
      else
        ITemp:=300;
      EndIf;
      
    If ((ITemp in [0..120]) and (RTemp in [0..120]))
      then
        DiffTemp:=TSoll-ITemp;
        DiffIR:=RTemp-ITemp;
      else
        DiffTemp:=0;
        DiffIR:=0;
      EndIf;

    If ((TSoll<=ITemp) and (TimeStart=false) and (ProgRun=true))
      then
        TimeStart:=true;
        Resume(Timer);
      EndIf;
      
    LCDXY(0,1);
    Write(LCDOut,IntToStr(ITemp:3)+' '+IntToStr(RTemp:3)+'  '+ByteToStr(Speed:3)+
                 ' '+ByteToStr(H1:3)+' '+ByteToStr(H2:3));
  End ShowStatus;


Procedure Start;                             { Start des Programmes            }
  Var Counter :Byte;
  Begin
    For Counter:=0 to 2 do
      SoftPWMarr[Counter]:=0;
      EndFor;
    ADCAct:=true;
    TSoll:=25;
    HeatTime:=1;
    HeatSec:=60;
    ProgRun:=false;
    NewTime:=false;
    
    LCDCursor(false, false);                 { Cursor & Unterstreichen aus     }
    LCDCharSet(#0, $18, $18, $06, $09, $08, $09, $06, $00);     // C
    LCDCharSet(#1, $00, $10, $08, $04, $02, $01, $00, $00);     // \
    LCDCharSet(#2, $00, $0E, $15, $15, $13, $0E, $00, $00);     // Uhr
    LCDCLR;
    LCDXY(0,0);
    Write(LCDOut,'* Klimakammer V1.2 *');
    LCDXY(0,1);
    Write(LCDOut,'*         PCB V1.0 *');
    LCDXY(0,2);
    Write(LCDOut,'*  (C)2008  by dp  *');
    LCDXY(0,3);
    Write(LCDOut,'**** CrazyHarry ****');
    MDelay(2000);

    LCDCLR;
    LCDXY(0,0);
    Write(LCDOut,'TI'+#0+' TR'+#0+'  FR% H1% H2%');
    LCDXY(0,2);
    Write(LCDOut,'TS '+IntToStr(TSoll:3)+#0+'   Time '+ByteToStr(HeatTime:2)+'min');
    LCDXY(0,3);
    Write(LCDOut,'[ ]  -'+#0+' +'+#0+' -'+#2+' +'+#2+' Run');
  End Start;
  

Procedure SetValues;
  Begin
    If Inp_Raise_G(TempMinus)
      then
        If TSoll in [26..80]
          then
            Dec(TSoll);
          EndIf;
      EndIf;
      
    If Inp_Raise_G(TempPlus)
      then
        If TSoll in [25..79]
          then
            Inc(TSoll);
          EndIf;
      EndIf;
      
    If Inp_Raise_G(TimeMinus)
      then
        If HeatTime in [2..15]
          then
            Dec(HeatTime);
          EndIf;
      EndIf;
      
    If Inp_Raise_G(TimePlus)
      then
        If HeatTime in [1..14]
          then
            Inc(HeatTime);
          EndIf;
      EndIf;
    LCDXY(3,2);
    Write(LCDOut,IntToStr(TSoll:3));
    LCDXY(15,2);
    Write(LCDOut,ByteToStr(HeatTime:2));
  End SetValues;
  
  
Procedure Off;
  Begin
    ProgRun:=false;
    Suspend(Reg_Temp);
    Suspend(Timer);
    HeatControl1(0);
    LCDXY(15,2);
    Write(LCDOut,ByteToStr(HeatTime:2)+'min');
    LCDXY(1,3);
    Write(LCDOut,' ');
    LCDXY(17,3);
    Write(LCDOut,'Run');
    SwitchPort_G_Clear;
  End Off;


Procedure SetOnOff;
  Begin
    If Inp_Raise_G(OnOff)
      then
        If ProgRun                           // Ausschalten
          then
            Off;
          else                               // Einschalten
            ProgRun:=true;
            TimeStart:=false;
            HeatSec:=Word(HeatTime)*60;
            StopAuto:=false;
            NewTime:=true;
            Resume(Reg_Temp);
            LCDXY(17,3);
            Write(LCDOut,'Esc');
          EndIf;
      EndIf;
  End SetOnOff;
  

Procedure ShowTime;
  Begin
    If ((TimeStart=true) and (NewTime=true))
      then
        LCDXY(15,2);
        Write(LCDOut,IntToStr(HeatSec:3)+'s ');
        NewTime:=false;
      EndIf;
  End ShowTime;
  
  
Procedure Main;
  Begin
    If ProgRun
      then
        SetOnOff;
        If StopAuto=true
          then
            Off;
          EndIf;
        ShowTime;
      else
        SetOnOff;
        SetValues;
      EndIf;
    If ADCAct=true
      then
        ShowStatus;
        If ProgRun
          then
            ShowRun;
          EndIf;
        ADCAct:=false;
      EndIf;
  End Main;
{--------------------------------------------------------------}
{ Main Program }
{$IDATA}
Begin
  EnableInts;
  Start_Processes;
  Suspend(Reg_Temp);
  Suspend(Timer);
  Start;
  Loop
    Main;

  EndLoop;
End Klimakammer.

